@@ -1,7 +1,7 @@
 /**
  * @file common.c common routines for Liferea
  * 
- * Copyright (C) 2003-2006  Lars Lindner <lars.lindner@gmx.net>
+ * Copyright (C) 2003-2007  Lars Lindner <lars.lindner@gmail.com>
  * Copyright (C) 2004-2006  Nathan J. Conrad <t98502@users.sourceforge.net>
  * Copyright (C) 2004       Karl Soderstrom <ks@xanadunet.net>
  *
@@ -39,17 +39,10 @@
 
 #define _XOPEN_SOURCE /* glibc2 needs this (man strptime) */
 
-#include <libxml/xmlerror.h>
 #include <libxml/uri.h>
-#include <libxml/parser.h>
-#include <libxml/entities.h>
-#include <libxml/HTMLparser.h>
-#include <libxml/xpath.h>
 
 #include <glib.h>
-#include <glib/gstdio.h>
 #include <pango/pango-types.h>
-#include <gtk/gtk.h>
 
 #include <sys/stat.h>
 #include <errno.h>
@@ -58,19 +51,15 @@
 #include <time.h>
 #include <stdlib.h>
 #include <ctype.h>
-#include <unistd.h>	/* fsync */
 
-#include "support.h"
 #include "common.h"
 #include "conf.h"
+#include "e-date.h"
 #include "feed.h"
-#include "support.h"
 #include "debug.h"
 
 static gchar *lifereaUserPath = NULL;
 
-static void common_buffer_parse_error(void *ctxt, const gchar * msg, ...);
-
 /* Conversion function which should be applied to all read XML strings, 
    to ensure proper UTF8. This is because we use libxml2 in recovery
    mode which can produce invalid UTF-8. 
@@ -87,16 +76,16 @@ gchar * common_utf8_fix(xmlChar *string) {
 		/* if we have an invalid string we try to shorten
 		   it until it is valid UTF-8 */
 		debug0(DEBUG_PARSING, "parser delivered invalid UTF-8!");
-		debug1(DEBUG_PARSING, "	>>>%s<<<\n", string);
-		debug1(DEBUG_PARSING, "first invalid char is: >>>%s<<<\n" , invalid_offset);
+		debug1(DEBUG_PARSING, "	>>>%s<<<", string);
+		debug1(DEBUG_PARSING, "first invalid char is: >>>%s<<<" , invalid_offset);
 		debug0(DEBUG_PARSING, "removing invalid bytes");
 		
 		do {
 			memmove((void *)invalid_offset, invalid_offset + 1, strlen(invalid_offset + 1) + 1);			
 		} while(!g_utf8_validate(string, -1, &invalid_offset));
 		
-		debug0(DEBUG_PARSING, "result is:\n");
-		debug1(DEBUG_PARSING, "	>>>%s<<<\n", string);		
+		debug0(DEBUG_PARSING, "result is:");
+		debug1(DEBUG_PARSING, "	>>>%s<<<", string);
 	}
 	
 	return string;
@@ -113,489 +102,10 @@ long common_parse_long(gchar *str, long def) {
 	return num;
 }
 
-static xmlDocPtr common_parse_html(const gchar *html, gint len) {
-	xmlDocPtr out = NULL;
-	
-	g_assert(html != NULL);
-	g_assert(len >= 0);
-	
-
-	/* Note: NONET is not implemented so it will return an error
-	   because it doesn't know how to handle NONET. But, it might
-	   learn in the future. */
-	out = htmlReadMemory(html, len, NULL, "utf-8", HTML_PARSE_RECOVER | HTML_PARSE_NONET |
-	                     ((debug_level & DEBUG_HTML)?0:(HTML_PARSE_NOERROR | HTML_PARSE_NOWARNING)));
-	return out;
-}
-
-static xmlNodePtr common_html_doc_find_body(xmlDocPtr doc) {
-	xmlXPathContextPtr xpathCtxt = NULL;
-	xmlXPathObjectPtr xpathObj = NULL;
-	xmlNodePtr node = NULL;
-	xpathCtxt = xmlXPathNewContext(doc);
-	if(!xpathCtxt) goto error;
-
-	xpathObj = xmlXPathEvalExpression("/html/body", xpathCtxt);
-	if(!xpathObj) goto error;
-	if(!xpathObj->nodesetval->nodeMax) goto error;
-	
-	node = xpathObj->nodesetval->nodeTab[0];
- error:
-	if (xpathObj) xmlXPathFreeObject(xpathObj);
-	if (xpathCtxt) xmlXPathFreeContext(xpathCtxt);
-	return node;
-}
-
-/* Extract XHTML from the children of the passed node. */
-gchar * extractHTMLNode(xmlNodePtr cur, gint xhtmlMode, const gchar *defaultBase) {
-	xmlBufferPtr	buf;
-	xmlChar         *xml_base = NULL;
-	gchar		*result = NULL;
-
-	/* Create the new document and add the div tag*/
-	xmlDocPtr newDoc = xmlNewDoc( BAD_CAST "1.0" );;
-	xmlNodePtr divNode = xmlNewNode(NULL, BAD_CAST "div");;
-	xmlDocSetRootElement( newDoc, divNode );
-	xmlNewNs(divNode, BAD_CAST "http://www.w3.org/1999/xhtml", NULL);
-
-	/* Set the xml:base  of the div tag */
-	xml_base = xmlNodeGetBase(cur->doc, cur);
-	if(xml_base) {
-		xmlNodeSetBase( divNode, xml_base );
-		xmlFree(xml_base);
-	}
-	else if(defaultBase)
-		xmlNodeSetBase( divNode, defaultBase);
-	
-	if(xhtmlMode == 0) { /* Read escaped HTML and convert to XHTML, placing in a div tag */
-		xmlDocPtr oldDoc;
-		xmlNodePtr copiedNodes = NULL;
-		xmlChar *escapedhtml;
-		
-		/* Parse the HTML into oldDoc*/
-		escapedhtml = xmlNodeListGetString(cur->doc, cur->xmlChildrenNode, 1);//xmlNodeDump(tmpBuf, cur->doc, cur, 0, 0);
-		if(escapedhtml) {
-			escapedhtml = g_strstrip(escapedhtml);	/* stripping whitespaces to make empty string detection easier */
-			if(*escapedhtml) {			/* never process empty content, xmlDocCopy() doesn't like it... */
-				xmlNodePtr body;
-				oldDoc = common_parse_html(escapedhtml, strlen(escapedhtml));
-				body = common_html_doc_find_body(oldDoc);
-
-				if(body) {
-					/* Copy in the html tags */
-					copiedNodes = xmlDocCopyNodeList(newDoc, body->xmlChildrenNode);
-					// FIXME: is the above correct? Why only operate on the first child node?
-					// It might be unproblematic because all content is wrapped in a <div>...
-					xmlAddChildList(divNode, copiedNodes);
-				}
-				xmlFreeDoc(oldDoc);
-				xmlFree(escapedhtml);
-			}
-		}
-	} else if(xhtmlMode == 1 || xhtmlMode == 2){ /* Read multiple XHTML tags and embed in div tag */
-		xmlNodePtr copiedNodes = xmlDocCopyNodeList( newDoc, cur->xmlChildrenNode);
-		xmlAddChildList(divNode, copiedNodes);
-	}
-	
-	buf = xmlBufferCreate();
-	xmlNodeDump(buf, newDoc, xmlDocGetRootElement(newDoc), 0, 0 );
-	
-	if(xmlBufferLength(buf) > 0)
-		result = xmlCharStrdup(xmlBufferContent(buf));
-
-	xmlBufferFree(buf);
-	xmlFreeDoc(newDoc);
-	return result;
-}
-
-gchar * common_strip_dhtml(const gchar *html) {
-	gchar *tmp;
-
-	// FIXME: move to XSLT stylesheet that post processes
-	// generated XHTML. The solution below might break harmless
-	// escaped HTML.
-	
-	/* remove some nasty DHTML stuff from the given HTML content */
-	tmp = g_strdup(html);
-	tmp = common_strreplace(tmp, " onload=", " no_onload=");
-	tmp = common_strreplace(tmp, " onLoad=", " no_onLoad=");
-	tmp = common_strreplace(tmp, "script>", "no_script>");		
-	tmp = common_strreplace(tmp, "<script ", "<no_script ");
-	tmp = common_strreplace(tmp, "<meta ", "<no_meta ");
-	tmp = common_strreplace(tmp, "/meta>", "/no_meta>");
-	tmp = common_strreplace(tmp, "<iframe ", "<no_iframe ");
-	tmp = common_strreplace(tmp, "/iframe>", "/no_iframe>");
-	
-	return tmp;
-}
-
-/* Convert the given string to proper XHTML content.*/
-gchar * common_text_to_xhtml(const gchar *sourceText) {
-	gchar		*text, *result = NULL;
-	xmlDocPtr	doc = NULL;
-	xmlBufferPtr	buf;
-	xmlNodePtr 	body, cur;
-	
-	if(!sourceText)
-		return g_strdup("");
-	
-	text = g_strstrip(g_strdup(sourceText));	/* stripping whitespaces to make empty string detection easier */
-	if(*text) {	
-		doc = common_parse_html(text, strlen(text));
-		
-		buf = xmlBufferCreate();
-		body = common_html_doc_find_body(doc);
-		if(body) {
-			cur = body->xmlChildrenNode;
-			while(cur) {
-				xmlNodeDump(buf, doc, cur, 0, 0 );
-				cur = cur->next;
-			}
-		}
-
-		if(xmlBufferLength(buf) > 0)
-			result = xmlCharStrdup(xmlBufferContent(buf));
-		else
-			result = g_strdup("");
-
-		xmlBufferFree(buf);
-		xmlFreeDoc(doc);
-		g_free(text);
-	} else {
-		result = text;
-	}
-	
-	return result;
-}
-
-gboolean common_is_well_formed_xhtml(const gchar *data) {
-	gchar		*xml;
-	gboolean	result;
-	errorCtxtPtr	errors;
-	xmlDocPtr	doc;
-
-	if(!data)
-		return FALSE;
-	
-	errors = g_new0(struct errorCtxt, 1);
-	errors->msg = g_string_new(NULL);
-
-	xml = g_strdup_printf("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n\n<test>%s</test>", data);
-	
-	doc = common_parse_xml(xml, strlen(xml), FALSE, errors);
-	if(doc)
-		xmlFreeDoc(doc);
-		
-	g_free(xml);
-	g_string_free(errors->msg, TRUE);
-	result = (0 == errors->errorCount);
-	g_free(errors);
-	
-	return result;
-}
-
-typedef struct {
-	gchar	*data;
-	gint	length;
-} result_buffer;
-
-static void unhtmlizeHandleCharacters (void *user_data, const xmlChar *string, int length) {
-	result_buffer	*buffer = (result_buffer *)user_data;
-	gint 		old_length;
-
-	old_length = buffer->length;	
-	buffer->length += length;
-	buffer->data = g_renew(gchar, buffer->data, buffer->length + 1);
-        strncpy(buffer->data + old_length, (gchar *)string, length);
-	buffer->data[buffer->length] = 0;
-
-}
-
-static void _unhtmlize(gchar *string, result_buffer *buffer) {
-	htmlParserCtxtPtr	ctxt;
-	htmlSAXHandlerPtr	sax_p = g_new0(htmlSAXHandler, 1);
- 	sax_p->characters = unhtmlizeHandleCharacters;
-	ctxt = htmlCreatePushParserCtxt(sax_p, buffer, string, strlen(string), "", XML_CHAR_ENCODING_UTF8);
-	htmlParseChunk(ctxt, string, 0, 1);
-	htmlFreeParserCtxt(ctxt);
- 	g_free(sax_p);
-}
-
-static void _unxmlize(gchar *string, result_buffer *buffer) {
-	xmlParserCtxtPtr	ctxt;
-	xmlSAXHandler	*sax_p = g_new0(xmlSAXHandler, 1);
- 	sax_p->characters = unhtmlizeHandleCharacters;
-	ctxt = xmlCreatePushParserCtxt(sax_p, buffer, string, strlen(string), "");
-	xmlParseChunk(ctxt, string, 0, 1);
-	xmlFreeParserCtxt(ctxt);
- 	g_free(sax_p);
-}
-
-/* Converts a UTF-8 strings containing any XML stuff to 
-   a string without any entities or tags containing all
-   text nodes of the given HTML string. The original 
-   string will be freed. */
-static gchar * unmarkupize(gchar *string, void(*parse)(gchar *string, result_buffer *buffer)) {
-	gchar			*result;
-	result_buffer		*buffer;
-	
-	if(NULL == string)
-		return NULL;
-		
-	string = common_utf8_fix(string);
-
-	/* only do something if there are any entities or tags */
-	if(NULL == (strpbrk(string, "&<>")))
-		return string;
-
-	buffer = g_new0(result_buffer, 1);
-	parse(string, buffer);
-	result = buffer->data;
-	g_free(buffer);
- 
- 	if(result == NULL || !g_utf8_strlen(result, -1)) {
- 		/* Something went wrong in the parsing.
- 		 * Use original string instead */
-		g_free(result);
- 		return string;
- 	} else {
- 		g_free(string);
- 		return result;
- 	}
-}
-
-gchar * unhtmlize(gchar * string) { return unmarkupize(string, _unhtmlize); }
-gchar * unxmlize(gchar * string) { return unmarkupize(string, _unxmlize); }
-
-#define MAX_PARSE_ERROR_LINES	10
-
-/**
- * Error buffering function to be registered by 
- * xmlSetGenericErrorFunc(). This function is called on
- * each libxml2 error output and collects all output as
- * HTML in the buffer ctxt points to. 
- *
- * @param	ctxt	error context
- * @param	msg	printf like format string 
- */
-static void common_buffer_parse_error(void *ctxt, const gchar * msg, ...) {
-	va_list		params;
-	errorCtxtPtr	errors = (errorCtxtPtr)ctxt;
-	gchar		*newmsg;
-	gchar		*tmp;
-	
-	if(MAX_PARSE_ERROR_LINES > errors->errorCount++) {
-		
-		va_start(params, msg);
-		newmsg = g_strdup_vprintf(msg, params);
-		va_end(params);
-
-		g_assert(NULL != newmsg);
-		newmsg = common_utf8_fix(newmsg);
-		tmp = g_markup_escape_text(newmsg, -1);
-		g_free(newmsg);
-		newmsg = tmp;
-	
-		g_string_append_printf(errors->msg, "<pre>%s</pre", newmsg);
-		g_free(newmsg);
-	}
-	
-	if(MAX_PARSE_ERROR_LINES == errors->errorCount)
-		g_string_append_printf(errors->msg, "<br />%s", _("[There were more errors. Output was truncated!]"));
-}
-
-static xmlDocPtr entities = NULL;
-
-/* and I thought writing such functions is evil... */
-xmlEntityPtr common_process_entities(void *ctxt, const xmlChar *name) {
-	xmlEntityPtr	entity, found;
-	xmlChar		*tmp;
-	
-	entity = xmlGetPredefinedEntity(name);
-	if(NULL == entity) {
-		if(NULL == entities) {
-			/* loading HTML entities */
-			entities = xmlNewDoc(BAD_CAST "1.0");
-			xmlCreateIntSubset(entities, BAD_CAST "HTML entities", NULL, PACKAGE_DATA_DIR "/" PACKAGE "/dtd/html.ent");
-			entities->extSubset = xmlParseDTD(entities->intSubset->ExternalID, entities->intSubset->SystemID);
-		}
-		
-		if(NULL != (found = xmlGetDocEntity(entities, name))) {
-			/* returning as faked predefined entity... */
-			tmp = xmlStrdup(found->content);
-			tmp = unhtmlize(tmp);	/* arghh ... slow... */
-			entity = (xmlEntityPtr)g_new0(xmlEntity, 1);
-			entity->type = XML_ENTITY_DECL;
-			entity->name = name;
-			entity->orig = NULL;
-			entity->content = tmp;
-			entity->length = g_utf8_strlen(tmp, -1);
-			entity->etype = XML_INTERNAL_PREDEFINED_ENTITY;
-		}
-	}
-	if(NULL == entity) {		
-		g_print("unsupported entity: %s\n", name);
-	}
-	return entity;
-}
-
-xmlNodePtr common_xpath_find(xmlNodePtr node, gchar *expr) {
-	xmlNodePtr	result = NULL;
-	
-	if(node && node->doc) {
-		xmlXPathContextPtr xpathCtxt = NULL;
-		xmlXPathObjectPtr xpathObj = NULL;
-		
-		if(NULL != (xpathCtxt = xmlXPathNewContext(node->doc))) {
-			xpathCtxt->node = node;
-			xpathObj = xmlXPathEval(expr, xpathCtxt);
-		}
-		
-		if(xpathObj && !xmlXPathNodeSetIsEmpty(xpathObj->nodesetval))
-			result = xpathObj->nodesetval->nodeTab[0];
-		
-		if(xpathObj) xmlXPathFreeObject(xpathObj);
-		if(xpathCtxt) xmlXPathFreeContext(xpathCtxt);
-	}
-	return result;
-}
-
-gboolean common_xpath_foreach_match(xmlNodePtr node, gchar *expr, xpathMatchFunc func, gpointer user_data) {
-	
-	if(node && node->doc) {
-		xmlXPathContextPtr xpathCtxt = NULL;
-		xmlXPathObjectPtr xpathObj = NULL;
-		
-		if(NULL != (xpathCtxt = xmlXPathNewContext(node->doc))) {
-			xpathCtxt->node = node;
-			xpathObj = xmlXPathEval(expr, xpathCtxt);
-		}
-			
-		if(xpathObj && xpathObj->nodesetval && xpathObj->nodesetval->nodeMax) {
-			int	i;
-			for(i = 0; i < xpathObj->nodesetval->nodeNr; i++)
-				(*func)(xpathObj->nodesetval->nodeTab[i], user_data);
-		}
-		
-		if(xpathObj) xmlXPathFreeObject(xpathObj);
-		if(xpathCtxt) xmlXPathFreeContext(xpathCtxt);
-		return TRUE;
-	}
-	return FALSE;
-}
-
-xmlDocPtr common_parse_xml(gchar *data, size_t length, gboolean recovery, errorCtxtPtr errCtx) {
-	xmlParserCtxtPtr	ctxt;
-	xmlDocPtr		doc;
-	
-	g_assert(NULL != data);
-
-	ctxt = xmlNewParserCtxt();
-	ctxt->sax->getEntity = common_process_entities;
-	
-	if(errCtx)
-		xmlSetGenericErrorFunc(errCtx, (xmlGenericErrorFunc)common_buffer_parse_error);
-	
-	doc = xmlSAXParseMemory(ctxt->sax, data, length, /* recovery = */ recovery);
-	
-	/* This seems to reset the errorfunc to its default, so that the
-	   GtkHTML2 module is not unhappy because it also tries to call the
-	   errorfunc on occasion. */
-	xmlSetGenericErrorFunc(NULL, NULL);
-	
-	xmlFreeParserCtxt(ctxt);
-	
-	return doc;
-}
-
-xmlDocPtr common_parse_xml_feed(feedParserCtxtPtr fpc) {
-	errorCtxtPtr		errors;
-		
-	g_assert(NULL != fpc->data);
-	g_assert(NULL != fpc->feed);
-	g_assert(NULL != fpc->itemSet);
-	
-	fpc->itemSet->valid = FALSE;
-	
-	/* we don't like no data */
-	if(0 == fpc->dataLength) {
-		debug1(DEBUG_PARSING, "common_parse_xml_feed(): empty input while parsing \"%s\"!", fpc->node->title);
-		g_string_append(fpc->feed->parseErrors, "Empty input!\n");
-		return NULL;
-	}
-
-	errors = g_new0(struct errorCtxt, 1);
-	errors->msg = fpc->feed->parseErrors;
-	
-	fpc->doc = common_parse_xml(fpc->data, (size_t)fpc->dataLength, fpc->recovery, errors);
-	if(!fpc->doc) {
-		debug1(DEBUG_PARSING, "xmlReadMemory: could not parse feed \"%s\"!\n", fpc->node->title);
-		g_string_prepend(fpc->feed->parseErrors, _("XML Parser: Could not parse document:\n"));
-		g_string_append(fpc->feed->parseErrors, "\n");
-	}
-
-	fpc->itemSet->valid = (errors->errorCount > 0);
-	g_free(errors);
-	
-	return fpc->doc;
-}
-
-/* The following slightly modified function was originally taken 
- * from GConf 2.16.0 backends/xml-dir.c: Copyright (C) 1999, 2000 Red Hat Inc. */
-
-/* for info on why this is used rather than xmlDocDump or xmlSaveFile
- * and friends, see http://bugzilla.gnome.org/show_bug.cgi?id=108329 */
-gint common_save_xml(xmlDocPtr doc, gchar *filename) {
-	FILE	*fp;
-	char	*xmlbuf;
-	int	fd, n;
-
-	fp = g_fopen(filename, "w");
-	if(NULL == fp)
-		return -1;
-  
-	xmlDocDumpFormatMemory(doc, (xmlChar **)&xmlbuf, &n, TRUE);
-	if(n <= 0) {
-		errno = ENOMEM;
-		return -1;
-	}
-
-	if(fwrite(xmlbuf, sizeof (xmlChar), n, fp) < n)	{
-		xmlFree (xmlbuf);
-		return -1;
-	}
-
-	xmlFree (xmlbuf);
-
-	/* From the fflush(3) man page:
-	*
-	* Note that fflush() only flushes the user space buffers provided by the
-	* C library. To ensure that the data is physically stored on disk the
-	* kernel buffers must be flushed too, e.g. with sync(2) or fsync(2).
-	*/
-
-	/* flush user-space buffers */
-	if (fflush (fp) != 0)
-		return -1;
-
-	if ((fd = fileno (fp)) == -1)
-		return -1;
-
-#ifdef HAVE_FSYNC
-	/* sync kernel-space buffers to disk */
-	if (fsync (fd) == -1)
-		return -1;
-#endif
-
-	fclose(fp);
-
-	return 0;
-}
-
 #define	TIMESTRLEN	256
 
 gchar * common_format_date(time_t date, const gchar *date_format) {
-	gchar		*result, *formbuf;
+	gchar		*result;
 	struct tm	date_tm;
 	
 	if (date == 0) {
@@ -605,9 +115,7 @@ gchar * common_format_date(time_t date, const gchar *date_format) {
 	localtime_r (&date, &date_tm);
 	
 	result = g_new0(gchar, TIMESTRLEN);
-	formbuf = g_locale_from_utf8 (date_format, -1, NULL, NULL, NULL);
-	strftime(result, TIMESTRLEN, formbuf, &date_tm);
-	g_free(formbuf);
+	e_utf8_strftime_fix_am_pm(result, TIMESTRLEN, date_format, &date_tm);
 	return result;
 }
 
@@ -616,7 +124,7 @@ gchar * common_format_nice_date(time_t date) {
 	time_t nowdate = time(NULL);
 	time_t yesdate;
 	struct tm then, now, yesterday;
-	gchar *temp, *buf, *formbuf;
+	gchar *temp, *buf;
 	gboolean done = FALSE;
 	
 	if (date == 0) {
@@ -629,7 +137,7 @@ gchar * common_format_nice_date(time_t date) {
 	localtime_r (&nowdate, &now);
 
 /*	if (nowdate - date < 60 * 60 * 8 && nowdate > date) {
-		strftime (buf, TIMESTRLEN, _("%l:%M %p"), &then);
+		e_utf8_strftime_fix_am_pm (buf, TIMESTRLEN, _("%l:%M %p"), &then);
 		done = TRUE;
 	}*/
 
@@ -638,9 +146,7 @@ gchar * common_format_nice_date(time_t date) {
 		    then.tm_mon == now.tm_mon &&
 		    then.tm_year == now.tm_year) {
 		    	/* translation hint: date format for today, reorder format codes as necessary */
-			formbuf = g_locale_from_utf8 (_("Today %l:%M %p"), -1, NULL, NULL, NULL);
-			strftime (buf, TIMESTRLEN, formbuf, &then);
-			g_free(formbuf);
+			e_utf8_strftime_fix_am_pm (buf, TIMESTRLEN, _("Today %l:%M %p"), &then);
 			done = TRUE;
 		}
 	}
@@ -651,9 +157,7 @@ gchar * common_format_nice_date(time_t date) {
 		    then.tm_mon == yesterday.tm_mon &&
 		    then.tm_year == yesterday.tm_year) {
 		    	/* translation hint: date format for yesterday, reorder format codes as necessary */
-			formbuf = g_locale_from_utf8 (_("Yesterday %l:%M %p"), -1, NULL, NULL, NULL);
-			strftime (buf, TIMESTRLEN, formbuf, &then);
-			g_free(formbuf);
+			e_utf8_strftime_fix_am_pm (buf, TIMESTRLEN, _("Yesterday %l:%M %p"), &then);
 			done = TRUE;
 		}
 	}
@@ -666,9 +170,7 @@ gchar * common_format_nice_date(time_t date) {
 			    then.tm_mon == yesterday.tm_mon &&
 			    then.tm_year == yesterday.tm_year) {
 			    	/* translation hint: date format for dates older than 2 days but not older than a week, reorder format codes as necessary */
-				formbuf = g_locale_from_utf8 (_("%a %l:%M %p"), -1, NULL, NULL, NULL);
-				strftime (buf, TIMESTRLEN, formbuf, &then);
-				g_free(formbuf);
+				e_utf8_strftime_fix_am_pm (buf, TIMESTRLEN, _("%a %l:%M %p"), &then);
 				done = TRUE;
 				break;
 			}
@@ -677,14 +179,10 @@ gchar * common_format_nice_date(time_t date) {
 	if (!done) {
 		if (then.tm_year == now.tm_year) {
 			/* translation hint: date format for dates older than a week but from this year, reorder format codes as necessary */
-			formbuf = g_locale_from_utf8 (_("%b %d %l:%M %p"), -1, NULL, NULL, NULL);
-			strftime (buf, TIMESTRLEN, formbuf, &then);
-			g_free(formbuf);
+			e_strftime_fix_am_pm (buf, TIMESTRLEN, _("%b %d %l:%M %p"), &then);
 		} else {
 			/* translation hint: date format for dates from the last years, reorder format codes as necessary */
-			formbuf = g_locale_from_utf8 (_("%b %d %Y"), -1, NULL, NULL, NULL);
-			strftime (buf, TIMESTRLEN, formbuf, &then);
-			g_free(formbuf);
+			e_strftime_fix_am_pm (buf, TIMESTRLEN, _("%b %d %Y"), &then);
 		}
 	}
 
@@ -759,10 +257,10 @@ time_t parseISO8601Date(gchar *date) {
 			
 			return t;
 		} else {
-			debug0(DEBUG_PARSING, "internal error! time conversion error! mktime failed!\n");
+			debug0(DEBUG_PARSING, "internal error! time conversion error! mktime failed!");
 		}
 	} else {
-		debug0(DEBUG_PARSING, "Invalid ISO8601 date format! Ignoring <dc:date> information!\n");
+		debug0(DEBUG_PARSING, "Invalid ISO8601 date format! Ignoring <dc:date> information!");
 	}
 	
 	return 0;
@@ -876,7 +374,7 @@ time_t parseRFC822Date(gchar *date) {
 			t = t - (t2 - t);
 			return t;
 		} else {
-			debug0(DEBUG_PARSING, "internal error! time conversion error! mktime failed!\n");
+			debug0(DEBUG_PARSING, "internal error! time conversion error! mktime failed!");
 		}
 	}
 	
@@ -896,7 +394,7 @@ static void common_check_dir(gchar *path) {
 static void common_init_cache_path(void) {
 	gchar *cachePath;
 
-	lifereaUserPath = g_strdup_printf("%s" G_DIR_SEPARATOR_S ".liferea_1.2", g_get_home_dir());
+	lifereaUserPath = g_strdup_printf("%s" G_DIR_SEPARATOR_S ".liferea_1.4", g_get_home_dir());
 	cachePath = g_strdup_printf("%s" G_DIR_SEPARATOR_S "cache", lifereaUserPath);
 
 	common_check_dir(g_strdup(lifereaUserPath));
@@ -931,6 +429,65 @@ gchar * common_create_cache_filename(const gchar *folder, const gchar *filename,
 	return result;
 }
 
+const gchar * common_http_error_to_str(gint httpstatus) {
+	gchar	*tmp = NULL;
+	
+	switch(httpstatus) {
+		case 401:tmp = _("You are unauthorized to download this feed. Please update your username and "
+		                 "password in the feed properties dialog box.");break;
+		case 402:tmp = _("Payment Required");break;
+		case 403:tmp = _("Access Forbidden");break;
+		case 404:tmp = _("Resource Not Found");break;
+		case 405:tmp = _("Method Not Allowed");break;
+		case 406:tmp = _("Not Acceptable");break;
+		case 407:tmp = _("Proxy Authentication Required");break;
+		case 408:tmp = _("Request Time-Out");break;
+		case 410:tmp = _("Gone. Resource doesn't exist. Please unsubscribe!");break;
+	}
+	
+	if(!tmp) {
+		switch(httpstatus / 100) {
+			case 3:tmp = _("Feed not available: Server requested unsupported redirection!");break;
+			case 4:tmp = _("Client Error");break;
+			case 5:tmp = _("Server Error");break;
+			default:tmp = _("(unknown networking error happened)");break;
+		}
+	}
+	
+	return tmp;
+}
+
+const gchar * common_netio_error_to_str(gint netstatus) {
+	gchar	*tmp = NULL;
+	
+	switch(netstatus) {
+		case NET_ERR_URL_INVALID:    tmp = _("URL is invalid"); break;
+		case NET_ERR_PROTO_INVALID:  tmp = _("Unsupported network protocol"); break;
+		case NET_ERR_UNKNOWN:
+		case NET_ERR_CONN_FAILED:
+		case NET_ERR_SOCK_ERR:       tmp = _("Error connecting to remote host"); break;
+		case NET_ERR_HOST_NOT_FOUND: tmp = _("Hostname could not be found"); break;
+		case NET_ERR_CONN_REFUSED:   tmp = _("Network connection was refused by the remote host"); break;
+		case NET_ERR_TIMEOUT:        tmp = _("Remote host did not finish sending data"); break;
+		/* Transfer errors */
+		case NET_ERR_REDIRECT_COUNT_ERR: tmp = _("Too many HTTP redirects were encountered"); break;
+		case NET_ERR_REDIRECT_ERR:
+		case NET_ERR_HTTP_PROTO_ERR: 
+		case NET_ERR_GZIP_ERR:           tmp = _("Remote host sent an invalid response"); break;
+		/* These are handled by HTTP error codes...
+		   case NET_ERR_HTTP_410:
+		   case NET_ERR_HTTP_404:
+		   case NET_ERR_HTTP_NON_200:
+		*/
+		case NET_ERR_AUTH_FAILED:
+		case NET_ERR_AUTH_NO_AUTHINFO: tmp = _("Authentication failed"); break;
+		case NET_ERR_AUTH_GEN_AUTH_ERR:
+		case NET_ERR_AUTH_UNSUPPORTED: tmp = _("Webserver's authentication method incompatible with Liferea"); break;
+	}
+	
+	return tmp;
+}
+
 static gchar * byte_to_hex(unsigned char nr) {
 	gchar *result = NULL;
 
@@ -1030,6 +587,66 @@ xmlChar * common_build_url(const gchar *url, const gchar *baseURL) {
 	return absURL;
 }
 
+/* The following three functions are to be used for
+   pixmap handling as provided by glade-2 in support.c */
+
+static GList *pixmaps_directories = NULL;
+
+/* Use this function to set the directory containing installed pixmaps. */
+void
+add_pixmap_directory (const gchar     *directory)
+{
+  pixmaps_directories = g_list_prepend (pixmaps_directories,
+                                        g_strdup (directory));
+}
+
+/* This is an internally used function to find pixmap files. */
+static gchar*
+find_pixmap_file (const gchar     *filename)
+{
+	GList *elem;
+
+	/* We step through each of the pixmaps directory to find it. */
+	elem = pixmaps_directories;
+	while (elem) {
+		gchar *pathname = g_strdup_printf ("%s%s%s", (gchar*)elem->data,
+                                	 G_DIR_SEPARATOR_S, filename);
+		if (g_file_test (pathname, G_FILE_TEST_EXISTS))
+			return pathname;
+		g_free (pathname);
+		elem = elem->next;
+	}
+	return NULL;
+}
+
+/* This is an internally used function to create pixmaps. */
+GdkPixbuf *
+create_pixbuf (const gchar *filename)
+{
+	gchar *pathname = NULL;
+	GdkPixbuf *pixbuf;
+	GError *error = NULL;
+
+	if (!filename || !filename[0])
+		return NULL;
+
+	pathname = find_pixmap_file (filename);
+
+	if (!pathname) {
+		g_warning (_("Couldn't find pixmap file: %s"), filename);
+		return NULL;
+	}
+
+	pixbuf = gdk_pixbuf_new_from_file (pathname, &error);
+	if (!pixbuf) {
+		fprintf (stderr, "Failed to load pixbuf file: %s: %s\n",
+		       pathname, error->message);
+		g_error_free (error);
+	}
+	g_free (pathname);
+	return pixbuf;
+}
+
 const gchar * common_get_direction_mark(gchar *text) {
 	PangoDirection		pango_direction = PANGO_DIRECTION_NEUTRAL;
 	GtkTextDirection	gtk_direction;
