@@ -28,7 +28,7 @@
 #include <time.h>
 
 /* Currently Liferea knows only a single type of items used
-   for the itemset types feed, folder and vfolder. So each 
+   for the itemset types feed, folder and search folder. So each 
    feed list type provider must provide it's data using the
    item interface. */
  
@@ -38,12 +38,14 @@
 
 /**
  * An item stores a particular entry in a feed or a search.
- * Each item belongs to an item set. An itemset is a collection
- * of items. There are different item set types (e.g. feed,
- * folder,vfolder or plugin). Each item has a source node.
- * The item set node and the item source node is different
- * for folders and vfolders. */
+ *  Each item belongs to an item set. An itemset is a collection
+ *  of items. There are different item set types (e.g. feed,
+ *  folder,vfolder or plugin). Each item has a source node.
+ *  The item set node and the item source node is different
+ *  for folders and vfolders. */
 typedef struct item {
+	gulong		id;			/**< internally unique item id */
+
 	/* those fields should not be accessed directly. Accessors are provided. */
 	gboolean 	readStatus;		/**< TRUE if the item has been read */
 	gboolean	newStatus;		/**< TRUE if the item was downloaded and is to be counted by the tray icon */
@@ -53,52 +55,24 @@ typedef struct item {
 	gboolean	hasEnclosure;		/**< TRUE if this item has at least one enclosure */
 	gchar		*title;			/**< Title */
 	gchar		*source;		/**< URL to the post online */
+	gchar		*sourceId;		/**< "Unique" syndication item identifier, for example <guid> in RSS */
+	gboolean	validGuid;		/**< TRUE if id of this item is a GUID and can be used for duplicate detection */
 	gchar		*real_source_url;	/**< (optional) URL of the real source (e.g. if listed in search engine result) */
 	gchar		*real_source_title;	/**< (optional) title of the real source */
 	gchar		*description;		/**< XHTML string containing the item's description */
-	gchar		*id;			/**< "Unique" syndication item identifier, for example <guid> in RSS */
-	gboolean	validGuid;		/**< TRUE if id of this item is a GUID and can be used for duplicate detection */
 	
 	GSList		*metadata;		/**< Metadata of this item */
 	GHashTable	*tmpdata;		/**< Temporary data hash used during stateful parsing */
 	time_t		time;			/**< Last modified date of the headline */
-	
-	gulong		nr;			/**< Per item set unique item id */
-	struct itemSet	*itemSet;		/**< Pointer to the item set containing this item  */
-	gulong		sourceNr;		/**< Like nr but the number in the sourceNode item set (used for searches and vfolders) */
-	struct node	*sourceNode;		/**< Pointer to the source node of this item */
+
+	gchar		*commentFeedId;		/**< Id of the items comments feed (or NULL)*/
+	gboolean	isComment;		/**< TRUE if item is from a comment feed */
+
+	gchar		*nodeId;		/**< Node id the containing node */
+	gulong 		sourceNr;		/**< Either equal to nr or the number of the item this one is a copy of */
 
 } *itemPtr;
 
-/* Item Duplicate handling */
-
-/**
- * Adds the GUID of the given item to the GUID list.
- *
- * @param item		item with a valid GUID
- */
-void item_guid_list_add_id(itemPtr item);
-
-/**
- * Checks if there are items with the same GUID as the
- * given item. Must not be called after item_guid_list_add().
- *
- * @param item		item with a valid GUID
- *
- * @returns a list of all nodes that contains duplicates
- */
-GSList * item_guid_list_get_duplicates_for_id(itemPtr item);
-
-/**
- * Removes the GUID of the given item from the GUID list.
- * To be used upon item deletion.
- *
- * @param item		item with a valid GUID
- */
-void item_guid_list_remove_id(itemPtr item);
-
-/* Item handling */
-
 /**
  * Allocates a new item structure.
  *
@@ -106,6 +80,17 @@ void item_guid_list_remove_id(itemPtr item);
  */
 itemPtr 	item_new(void);
 
+/**
+ * Returns the item structure for the given item id or
+ * NULL if no such item does exist. The caller has to free
+ * the item with item_unload() once it is not used anymore.
+ *
+ * @param id	item id to load
+ *
+ * @returns item structure
+ */
+itemPtr		item_load(gulong id);
+
 /**
  * Method to create a copy of an item. The copy will be
  * linked to the original item to allow state update
@@ -126,9 +111,9 @@ const gchar * item_get_base_url(itemPtr item);
  * Free the memory used by an itempointer. The item needs to be
  * removed from the itemlist before calling this function.
  *
- * @param ip the item to remove
+ * @param item	the item to unload
  */
-void	item_free(itemPtr item);
+void	item_unload(itemPtr item);
 
 /* methods to access properties */
 /** Returns the id of item. */
@@ -157,26 +142,13 @@ void		item_set_real_source_title(itemPtr item, const gchar * source);
 /** Sets the item id */
 void		item_set_id(itemPtr item, const gchar * id);
 
-/**
- * Parse an xml tree and return a new item generated 
- * from the current node's information.
- *
- * @param cur		the XML node to parse
- * @param migrateCache	TRUE if cache migration requested
- *
- * @returns a new item structure
- */
-itemPtr item_parse_cache(xmlNodePtr cur, gboolean migrateCache);
-
 /**
  * Does the opposite of item_parse_cache. Adds an XML node
  * to the given feed item list node. 
  *
  * @param item		the item to save to cache
- * @param feedNode	the XML node to add to
- * @param rendering	TRUE if XML output is to be used
- *                  	for rendering (adds some more tags)
+ * @param parentNode	the XML node to add to
  */
-void item_to_xml(itemPtr item, xmlNodePtr feedNode, gboolean rendering);
+void item_to_xml(itemPtr item, xmlNodePtr parentNode);
 
 #endif
