@@ -1,7 +1,7 @@
 /**
  * @file metadata.h Metadata storage API
  *
- * Copyright (C) 2004 Nathan J. Conrad <t98502@users.sourceforge.net>
+ * Copyright (C) 2004-2006 Nathan J. Conrad <t98502@users.sourceforge.net>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -32,8 +32,8 @@ struct NsHandler;
 
 /** definition of various namespace tag handler */
 typedef void	(*registerNsFunc) (struct NsHandler *nsh, GHashTable *prefixhash, GHashTable *urihash);
-typedef void	(*parseChannelTagFunc)	(feedPtr fp, xmlNodePtr cur);
-typedef void	(*parseItemTagFunc)	(itemPtr ip, xmlNodePtr cur);
+typedef void	(*parseChannelTagFunc)	(feedParserCtxtPtr ctxt, xmlNodePtr cur);
+typedef void	(*parseItemTagFunc)	(feedParserCtxtPtr ctxt, xmlNodePtr cur);
 
 /** struct used to register namespace handler */
 typedef struct NsHandler {
@@ -48,51 +48,106 @@ typedef struct NsHandler {
 /* interface definitions for namespace rendering handler    */
 /* -------------------------------------------------------- */
 
-struct displayset;
-
-typedef void (*renderHTMLFunc)(gpointer data, struct displayset *displayset, gpointer user_data);
-
-/** Initialize the metadata subsystem */
-void metadata_init();
-
-/** Register a new type of metadata */
-void metadata_register(const gchar *strid, renderHTMLFunc renderfunc, gpointer user_data);
+/** 
+ * Initialize the metadata types to allow format checks.
+ */
+void metadata_init(void);
 
 /** 
  * Appends a value to the value list of a specific metadata type 
  * Don't mix this function with metadata_list_set() !
+ *
+ * @param metadata	the metadata list
+ * @param strid		the metadata type identifier
+ * @param data		data to add
+ *
+ * @returns the changed meta data list
  */
 GSList * metadata_list_append(GSList *metadata, const gchar *strid, const gchar *data);
 
 /** 
  * Sets (and overwrites if necessary) the value of a specific metadata type.
  * Don't mix this function with metadata_list_append() !
+ *
+ * @param metadata	the metadata list
+ * @param strid		the metadata type identifier
+ * @param data		data to add
  */
 void metadata_list_set(GSList **metadata, const gchar *strid, const gchar *data);
 
 /**
- * Returns a list of all values of a given type from a specified metadata list.
+ * Returns the first value of a given type from a specified metadata list.
+ * Do use this function only for single instance types.
  *
- * @returns a list of values
+ * @param metadata	the metadata list
+ * @param strid		the metadata type identifier
+ *
+ * @returns the first value (or NULL)
  */
-GSList * metadata_list_get(GSList *metadata, const gchar *strid);
-
-/**
- * Renders the given metadata list into the given display set.
- */
-void metadata_list_render(GSList *metadata, struct displayset *displayset);
+const gchar * metadata_list_get(GSList *metadata, const gchar *strid);
 
 /** 
- * Adds all elements from one metadata list to the other.
+ * Definition of metadata foreach function 
+ *
+ * @param key		metadata type id
+ * @param value		metadata value
+ * @param index		metadata list ordering index
+ * @param user_data	user data
+ */
+typedef void	(*metadataForeachFunc)	(const gchar *key, const gchar *value, guint index, gpointer user_data);
+
+/**
+ * Can be used to iterate over all key/value pairs of the given metadata list.
+ *
+ * @param metadata	the metadata list
+ * @param func		callback function
+ * @param user_data	data to be passed to func
+ */
+void metadata_list_foreach(GSList *metadata, metadataForeachFunc func, gpointer user_data);
+
+/**
+ * Returns a list of all values of a given type from a specified metadata list.
+ *
+ * @param metadata	the metadata list
+ * @param strid		the metadata type identifier
+ *
+ * @returns a list of values (or NULL)
+ */
+GSList * metadata_list_get_values(GSList *metadata, const gchar *strid);
+
+/** 
+ * Creates a copy of a given metadata list.
+ *
+ * @param metadata	the metadata list
  *
  * @returns the new list
  */
-GSList * metadata_list_copy(GSList *from, GSList *to);
+GSList * metadata_list_copy(GSList *list);
 
+/**
+ * Frees all memory allocated by the given metadata list.
+ *
+ * @param metadata	the metadata list
+ */
 void metadata_list_free(GSList *metadata);
 
+/**
+ * Adds the given metadata list to a given XML document node.
+ * To be used for saving feed metadata to cache.
+ *
+ * @param metadata	the metadata list
+ * @param parentNode	the XML node
+ */
 void metadata_add_xml_nodes(GSList *metadata, xmlNodePtr parentNode);
 
-GSList * metadata_parse_xml_nodes(xmlDocPtr, xmlNodePtr cur);
+/**
+ * Parses the given XML node and returns a new metadata attribute 
+ * value list. To be used for feed cache loading.
+ *
+ * @param cur	the XML node to parse
+ *
+ * @returns list of values
+ */
+GSList * metadata_parse_xml_nodes(xmlNodePtr cur);
 
 #endif
