@@ -1,7 +1,7 @@
 /**
  * @file mozsupport.cpp C++ portion of GtkMozEmbed support
  *
- * Copyright (C) 2004-2006 Lars Lindner <lars.lindner@gmx.net>
+ * Copyright (C) 2004-2007 Lars Lindner <lars.lindner@gmail.com>
  * Copyright (C) 2004-2006 Nathan J. Conrad <t98502@users.sourceforge.net>
  *
  * The preference handling was taken from the Galeon source
@@ -51,7 +51,8 @@ extern "C" {
 }
 
 extern "C" 
-gint mozsupport_key_press_cb(GtkWidget *widget, gpointer ev) {
+gint mozsupport_key_press_cb (GtkWidget *widget, gpointer ev)
+{
 	nsIDOMKeyEvent	*event = (nsIDOMKeyEvent*)ev;
 	PRUint32	keyCode = 0;
 	PRBool		alt, ctrl, shift;
@@ -62,15 +63,21 @@ gint mozsupport_key_press_cb(GtkWidget *widget, gpointer ev) {
 	   can be caught with the GTK code in ui_mainwindow.c.
 	   This is a bad case of code duplication... */
 	
-	event->GetCharCode(&keyCode);
-	if(keyCode == nsIDOMKeyEvent::DOM_VK_SPACE) {
-     		event->GetShiftKey(&shift);
-     		event->GetCtrlKey(&ctrl);
-     		event->GetAltKey(&alt);
-		if((1 == getNumericConfValue(BROWSE_KEY_SETTING)) &&
-		   !(alt | shift | ctrl)) {
-			if(mozsupport_scroll_pagedown(widget) == FALSE)
-				on_next_unread_item_activate(NULL, NULL);
+	event->GetCharCode (&keyCode);
+	if (keyCode == nsIDOMKeyEvent::DOM_VK_SPACE) {
+     		event->GetShiftKey (&shift);
+     		event->GetCtrlKey (&ctrl);
+     		event->GetAltKey (&alt);
+
+		/* Do trigger scrolling if the skimming hotkey is 
+		  <Space> with a modifier. Other cases (Space+modifier)
+		  are handled in src/ui/ui_mainwindow.c and if we
+		  get <Space> with a modifier here it needs no extra
+		  handling. */
+		if ((0 == conf_get_int_value (BROWSE_KEY_SETTING)) &&
+		    !(alt | shift | ctrl)) {
+			if (mozsupport_scroll_pagedown(widget) == FALSE)
+				on_next_unread_item_activate (NULL, NULL);
 			return TRUE;
 		}
 	}	
