@@ -71,7 +71,8 @@ static gnutls_certificate_client_credentials xcred;
 
 #endif
 
-#include "common.h"
+#include "../debug.h"
+#include "../common.h"
 #include "conversions.h"
 #include "net-support.h"
 #include "zlib_interface.h"
@@ -204,8 +205,8 @@ int NetPoll (struct feed_request * cur_ptr, int my_socket, int rw) {
 
 /* This assumes that a line of the header will be no longer than half the receive buffer */
 static int NetReadHeaderLine(int my_socket, void *proto_data, enum netio_proto proto,
-							 struct feed_request * cur_ptr, char *recvbuf, size_t recvbuflen,
-							 size_t *recvbufused, char **nextstr, char *headerline) {
+                             struct feed_request * cur_ptr, char *recvbuf, size_t recvbuflen,
+                             size_t *recvbufused, char **nextstr, char *headerline) {
 	int i;
 	char *ci;
 	/* Look for line ending */
@@ -408,7 +409,7 @@ static int NetConnect (int * my_socket, char * host, struct feed_request * cur_p
 	if (cur_ptr->netio_error != NET_ERR_OK)
 		return -1;
 	
-	debug1(DEBUG_NET, "successfully connected socket %d", my_socket);		
+	debug1(DEBUG_NET, "successfully connected socket %d", *my_socket);
 	
 #else
 	struct sockaddr_in address;	
@@ -497,13 +498,13 @@ static int NetConnect (int * my_socket, char * host, struct feed_request * cur_p
 		}
 	}
 	
-	debug1(DEBUG_NET, "successfully connected socket %d", my_socket);	
+	debug1(DEBUG_NET, "successfully connected socket %d", *my_socket);
 	
 	g_free(realhost);
 #endif /* HAVE_GETADDRINFO */
 #ifdef HAVE_GNUTLS
 	if (proto == NETIO_PROTO_HTTPS) {
-		debug1(DEBUG_NET, "setting up SSL session for socket %d", my_socket);
+		debug1(DEBUG_NET, "setting up SSL session for socket %d", *my_socket);
 		*proto_data = NetConnectGnutls(my_socket);
 		if (proto_data == NULL || *my_socket == -1)
 			return -1;
@@ -1158,7 +1159,7 @@ char * NetIO (char * host, char * url, struct feed_request * cur_ptr, char * aut
 /* Returns allocated string with body of webserver reply.
    Various status info put into struct feed_request * cur_ptr.
    Set suppressoutput=1 to disable ncurses calls. */
-static char * DownloadFeed (char * url, struct feed_request * cur_ptr, int suppressoutput) {
+char * DownloadFeed (char * url, struct feed_request * cur_ptr, int suppressoutput) {
 	char *host;					/* Needs to freed. */
 	char *tmphost;
 	char *freeme;
@@ -1230,85 +1231,24 @@ static char * DownloadFeed (char * url, struct feed_request * cur_ptr, int suppr
 	return returndata;
 }
 
-/*-----------------------------------------------------------------------*/
-/* some Liferea specific code...					 */
-
-#include "../debug.h"
-#include "../update.h"
-#include "downloadlib.h"
-
-/* Downloads a feed and returns the data or NULL as return value.
-   The url of the has to be passed in the feed structure.
-   If the the webserver reports a permanent redirection, the
-   feed url will be modified and the old URL 'll be freed. The
-   request structure 'll also contain the HTTP status and the
-   last modified string.
- */
-
-void downloadlib_init() {
+void
+netio_init (void)
+{
 #ifdef HAVE_GNUTLS
 	gcry_control (GCRYCTL_SET_THREAD_CBS, &gcry_threads_pthread);
 	
-	gnutls_global_init();
-	gnutls_certificate_allocate_credentials(&xcred);
-	gnutls_certificate_set_x509_trust_file(xcred, "ca.pem", GNUTLS_X509_FMT_PEM);
+	gnutls_global_init ();
+	gnutls_certificate_allocate_credentials (&xcred);
+	gnutls_certificate_set_x509_trust_file (xcred, "ca.pem", GNUTLS_X509_FMT_PEM);
 #endif
 }
 
-void downloadlib_process_url(struct request *request) {
-	struct feed_request	*netioRequest;
-	gchar *oldurl = g_strdup(request->source);
+void
+netio_deinit (void)
+{
+#ifdef HAVE_GNUTLS
+	gnutls_certificate_free_credentials (xcred);
 	
-	debug1(DEBUG_UPDATE, "downloading %s", request->source);
-	
-	g_assert(request->data == NULL);
-	g_assert(request->contentType == NULL);
-
-	netioRequest = g_new0(struct feed_request, 1);
-	netioRequest->feedurl = request->source;
-
-	if(request->updateState) {
-		netioRequest->lastmodified = request->updateState->lastModified;
-		netioRequest->etag = request->updateState->etag;
-		netioRequest->cookies = g_strdup(request->updateState->cookies);
-	}
-		
-	netioRequest->problem = 0;
-	netioRequest->netio_error = 0;
-	netioRequest->no_proxy = request->options->dontUseProxy?1:0;
-	netioRequest->content_type = NULL;
-	netioRequest->contentlength = 0;
-	netioRequest->authinfo = NULL;
-	netioRequest->servauth = NULL;
-	netioRequest->lasthttpstatus = 0; /* This might, or might not mean something to someone */
-	
-	request->data = DownloadFeed(oldurl, netioRequest, 0);
-
-	g_free(oldurl);
-	if(request->data == NULL)
-		netioRequest->problem = 1;
-	request->size = netioRequest->contentlength;
-	request->httpstatus = netioRequest->lasthttpstatus;
-	request->returncode = netioRequest->netio_error;
-	request->source = netioRequest->feedurl;
-	if(request->updateState) {
-		request->updateState->lastModified = netioRequest->lastmodified;
-		netioRequest->lastmodified = NULL; /* we have transfered ownership of the pointer */
-		request->updateState->etag = netioRequest->etag;
-		netioRequest->etag = NULL;
-	}
-	request->contentType = netioRequest->content_type;
-	g_free(netioRequest->servauth);
-	g_free(netioRequest->authinfo);
-	g_free(netioRequest->cookies);
-	g_free(netioRequest->lastmodified);
-	g_free(netioRequest->etag);
-
-	debug4(DEBUG_UPDATE, "download result - HTTP status: %d, error: %d, netio error:%d, data: %d",
-	                     request->httpstatus, 
-			     netioRequest->problem, 
-			     netioRequest->netio_error, 
-			     request->data);
-	g_free(netioRequest);
-	return;
+	gnutls_global_deinit ();
+#endif
 }
