@@ -2,7 +2,7 @@
  * @file atom10.c Atom 1.0 Parser
  * 
  * Copyright (C) 2005-2006 Nathan Conrad <t98502@users.sourceforge.net>
- * Copyright (C) 2003-2006 Lars Lindner <lars.lindner@gmx.net>
+ * Copyright (C) 2003-2007 Lars Lindner <lars.lindner@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,13 +25,12 @@
 
 #include <sys/time.h>
 #include <string.h>
-#include <libxml/xmlmemory.h>
-#include <libxml/parser.h>
-#include "support.h"
+
 #include "conf.h"
 #include "common.h"
 #include "debug.h"
 #include "feed.h"
+#include "feedlist.h"
 #include "ns_dc.h"
 #include "ns_content.h"
 #include "ns_slash.h"
@@ -41,8 +40,10 @@
 #include "ns_blogChannel.h"
 #include "ns_cC.h"
 #include "ns_photo.h"
-#include "callbacks.h"
+#include "ns_wfw.h"
 #include "metadata.h"
+#include "subscription.h"
+#include "xml.h"
 #include "atom10.h"
 
 #define ATOM10_NS BAD_CAST"http://www.w3.org/2005/Atom"
@@ -127,24 +128,23 @@ static gchar* atom10_parse_content_construct(xmlNodePtr cur, feedParserCtxtPtr c
 		
 		/* This that need to be de-encoded and should not contain sub-tags.*/
 		if(type && (g_str_equal(type,"html") || !g_strcasecmp(type, "text/html"))) {
-			ret = common_utf8_fix(extractHTMLNode(cur, 0, NULL));
+			ret = common_utf8_fix(xhtml_extract (cur, 0, NULL));
 		} else if(!type || !strcmp(type, "text") || !strncasecmp(type, "text/",5)) {
 			gchar *tmp;
 			/* Assume that "text/ *" files can be directly displayed.. kinda stated in the RFC */
 			ret = common_utf8_fix(xmlNodeListGetString(cur->doc, cur->xmlChildrenNode, 1));
-			if(ret) {
-				g_strchug(g_strchomp(ret));
-
-				if(!type || !strcasecmp(type, "text"))
-					tmp = atom10_mark_up_text_content(ret);
-				else
-					tmp = g_markup_printf_escaped("<pre>%s</pre>", ret);
-				g_free(ret);
-				ret = tmp;
-			}
+			
+			g_strchug(g_strchomp(ret));
+			
+			if(!type || !strcasecmp(type, "text"))
+				tmp = atom10_mark_up_text_content(ret);
+			else
+				tmp = g_markup_printf_escaped("<pre>%s</pre>", ret);
+			g_free(ret);
+			ret = tmp;
 		} else if(!strcmp(type,"xhtml") || !g_strcasecmp(type, "application/xhtml+xml")) {
 			/* The spec says to only show the contents of the div tag that MUST be present */
-			ret = common_utf8_fix(extractHTMLNode(cur, 2, NULL));
+			ret = common_utf8_fix(xhtml_extract (cur, 2, NULL));
 		} else {
 			/* Do nothing on unsupported content types. This allows summaries to be used. */
 			ret = NULL;
@@ -184,12 +184,12 @@ static gchar* atom10_parse_text_construct(xmlNodePtr cur, gboolean htmlified) {
 			}
 		}
 	} else if (!strcmp(type, "html")) {
-		ret = common_utf8_fix(common_utf8_fix(extractHTMLNode(cur, 0, NULL)));
+		ret = common_utf8_fix(xhtml_extract (cur, 0, NULL));
 		if (!htmlified)
 			ret = unhtmlize(unxmlize(ret));
 	} else if(!strcmp(type, "xhtml")) {
 		/* The spec says to show the contents of the div tag that MUST be present */
-		ret = common_utf8_fix(extractHTMLNode(cur, 2, NULL));
+		ret = common_utf8_fix(xhtml_extract (cur, 2, NULL));
 		
 		if (!htmlified)
 			ret = unhtmlize(ret);
@@ -255,18 +255,21 @@ static gchar * atom10_parse_person_construct(xmlNodePtr cur) {
 	return tmp;
 }
 
+/* Note: this function is called for both item and feed context */
 static gchar* atom10_parse_link(xmlNodePtr cur, feedParserCtxtPtr ctxt, struct atom10ParserState *state) {
 	gchar *href, *alternate = NULL;
 	
-	if(href = common_utf8_fix(xmlGetNsProp(cur, BAD_CAST"href", NULL))) {
+	href = common_utf8_fix(xmlGetNsProp(cur, BAD_CAST"href", NULL));
+	if(href) {
 		xmlChar *baseURL = xmlNodeGetBase(cur->doc, cur);
-		gchar *url, *relation, *escTitle = NULL, *title;
+		gchar *url, *relation, *type, *escTitle = NULL, *title;
 		
 		if(!baseURL && ctxt->feed->htmlUrl && ctxt->feed->htmlUrl[0] != '|' &&
 		   strstr(ctxt->feed->htmlUrl, "://"))
 			baseURL = xmlStrdup(BAD_CAST(ctxt->feed->htmlUrl));
 		url = common_build_url(href, baseURL);
-		
+
+		type = common_utf8_fix(xmlGetNsProp(cur, BAD_CAST"type", NULL));		
 		relation = common_utf8_fix(xmlGetNsProp(cur, BAD_CAST"rel", NULL));
 		title = common_utf8_fix(xmlGetNsProp(cur, BAD_CAST"title", NULL));
 		if(title)
@@ -276,9 +279,17 @@ static gchar* atom10_parse_link(xmlNodePtr cur, feedParserCtxtPtr ctxt, struct a
 		
 		if(!xmlHasNsProp(cur, BAD_CAST"rel", NULL) || !relation || g_str_equal(relation, BAD_CAST"alternate"))
 			alternate = g_strdup(url);
-		else if(g_str_equal(relation, "enclosure")) {
-			ctxt->item->metadata = metadata_list_append(ctxt->item->metadata, "enclosure", url);
-			ctxt->item->hasEnclosure = TRUE;
+		else if(g_str_equal(relation, "replies")) {
+			if(!type || g_str_equal(type, BAD_CAST"application/atom+xml")) {
+				gchar *commentUri = common_build_url(url, feed_get_html_url(ctxt->feed));
+				if (ctxt->item)
+					metadata_list_set(&ctxt->item->metadata, "commentFeedUri", commentUri);
+			}
+		} else if(g_str_equal(relation, "enclosure")) {
+			if (ctxt->item) {
+				ctxt->item->metadata = metadata_list_append(ctxt->item->metadata, "enclosure", url);
+				ctxt->item->hasEnclosure = TRUE;
+			}
 		} else {
 			/* FIXME: Maybe do something with other links such as "related" and add metadata for "via"? */;
 		}
@@ -287,6 +298,7 @@ static gchar* atom10_parse_link(xmlNodePtr cur, feedParserCtxtPtr ctxt, struct a
 		g_free(escTitle);
 		g_free(url);
 		g_free(relation);
+		g_free(type);
 		g_free(href);
 	} else {
 		/* FIXME: @href is required, this document is not valid Atom */;
@@ -298,7 +310,8 @@ static gchar* atom10_parse_link(xmlNodePtr cur, feedParserCtxtPtr ctxt, struct a
 static void atom10_parse_entry_author(xmlNodePtr cur, feedParserCtxtPtr ctxt, struct atom10ParserState *state) {
 	gchar *author;
 	
-	if(author = atom10_parse_person_construct(cur)) {
+	author = atom10_parse_person_construct(cur);
+	if(author) {
 		ctxt->item->metadata = metadata_list_append(ctxt->item->metadata, "author", author);
 		g_free(author);
 	}
@@ -324,7 +337,8 @@ static void atom10_parse_entry_category(xmlNodePtr cur, feedParserCtxtPtr ctxt,
 static void atom10_parse_entry_content(xmlNodePtr cur, feedParserCtxtPtr ctxt, struct atom10ParserState *state) {
 	gchar *content;
 	
-	if(content = atom10_parse_content_construct(cur, ctxt)) {
+	content = atom10_parse_content_construct(cur, ctxt);
+	if(content) {
 		item_set_description(ctxt->item, content);
 		g_free(content);
 	}
@@ -333,7 +347,8 @@ static void atom10_parse_entry_content(xmlNodePtr cur, feedParserCtxtPtr ctxt, s
 static void atom10_parse_entry_contributor(xmlNodePtr cur, feedParserCtxtPtr ctxt, struct atom10ParserState *state) {
 	gchar *contributor;
 	
-	if(contributor = atom10_parse_person_construct(cur)) {
+	contributor = atom10_parse_person_construct(cur);
+	if(contributor) {
 		ctxt->item->metadata = metadata_list_append(ctxt->item->metadata, "contributor", contributor);
 		g_free(contributor);
 	}
@@ -342,7 +357,8 @@ static void atom10_parse_entry_contributor(xmlNodePtr cur, feedParserCtxtPtr ctx
 static void atom10_parse_entry_id(xmlNodePtr cur, feedParserCtxtPtr ctxt, struct atom10ParserState *state) {
 	gchar *id;
 	
-	if(id = common_utf8_fix(xmlNodeListGetString(cur->doc, cur->xmlChildrenNode, 1))) {
+	id = common_utf8_fix(xmlNodeListGetString(cur->doc, cur->xmlChildrenNode, 1));
+	if(id) {
 		item_set_id(ctxt->item, id);
 		ctxt->item->validGuid = TRUE;
 		g_free(id);
@@ -352,7 +368,8 @@ static void atom10_parse_entry_id(xmlNodePtr cur, feedParserCtxtPtr ctxt, struct
 static void atom10_parse_entry_link(xmlNodePtr cur, feedParserCtxtPtr ctxt, struct atom10ParserState *state) {
 	gchar *href;
 	
-	if(href = atom10_parse_link(cur, ctxt, state)) {
+	href = atom10_parse_link(cur, ctxt, state);
+	if(href) {
 		item_set_source(ctxt->item, href);
 		g_free(href);
 	}
@@ -361,7 +378,8 @@ static void atom10_parse_entry_link(xmlNodePtr cur, feedParserCtxtPtr ctxt, stru
 static void atom10_parse_entry_published(xmlNodePtr cur, feedParserCtxtPtr ctxt, struct atom10ParserState *state) {
 	gchar *datestr;
 	
-	if(datestr = common_utf8_fix(xmlNodeListGetString(cur->doc, cur->xmlChildrenNode, 1))) {
+	datestr = common_utf8_fix(xmlNodeListGetString(cur->doc, cur->xmlChildrenNode, 1));
+	if(datestr) {
 		ctxt->item->time = parseISO8601Date(datestr);
 		ctxt->item->metadata = metadata_list_append(ctxt->item->metadata, "pubDate", datestr);
 		g_free(datestr);
@@ -371,7 +389,8 @@ static void atom10_parse_entry_published(xmlNodePtr cur, feedParserCtxtPtr ctxt,
 static void atom10_parse_entry_rights(xmlNodePtr cur, feedParserCtxtPtr ctxt, struct atom10ParserState *state) {
 	gchar *rights;
 
-	if(rights = atom10_parse_text_construct(cur, FALSE)) {
+	rights = atom10_parse_text_construct(cur, FALSE);
+	if(rights) {
 		ctxt->item->metadata = metadata_list_append(ctxt->item->metadata, "copyright", rights);
 		g_free(rights);
 	}
@@ -380,10 +399,10 @@ static void atom10_parse_entry_rights(xmlNodePtr cur, feedParserCtxtPtr ctxt, st
 /* <summary> can be used for short text descriptions, if there is no
    <content> description we show the <summary> content */
 static void atom10_parse_entry_summary(xmlNodePtr cur, feedParserCtxtPtr ctxt, struct atom10ParserState *state) {
-	gchar *summary;
-	
+		
 	if(!item_get_description(ctxt->item)) {
-		if(summary = atom10_parse_text_construct(cur, TRUE)) {
+		gchar *summary = atom10_parse_text_construct(cur, TRUE);
+		if(summary) {
 			item_set_description(ctxt->item, summary);
 			g_free(summary);
 		}
@@ -394,7 +413,8 @@ static void atom10_parse_entry_summary(xmlNodePtr cur, feedParserCtxtPtr ctxt, s
 static void atom10_parse_entry_title(xmlNodePtr cur, feedParserCtxtPtr ctxt, struct atom10ParserState *state) {
 	gchar *title;
 	
-	if(title = common_utf8_fix(atom10_parse_text_construct(cur, FALSE))) {
+	title = common_utf8_fix(atom10_parse_text_construct(cur, FALSE));
+	if(title) {
 		item_set_title(ctxt->item, title);
 		g_free(title);
 	}
@@ -403,7 +423,8 @@ static void atom10_parse_entry_title(xmlNodePtr cur, feedParserCtxtPtr ctxt, str
 static void atom10_parse_entry_updated(xmlNodePtr cur, feedParserCtxtPtr ctxt, struct atom10ParserState *state) {
 	gchar *datestr;
 	
-	if(datestr = common_utf8_fix(xmlNodeListGetString(cur->doc, cur->xmlChildrenNode, 1))) {
+	datestr = common_utf8_fix(xmlNodeListGetString(cur->doc, cur->xmlChildrenNode, 1));
+	if(datestr) {
 		ctxt->item->time = parseISO8601Date(datestr);
 		ctxt->item->metadata = metadata_list_append(ctxt->item->metadata, "contentUpdateDate", datestr);
 		g_free(datestr);
@@ -450,7 +471,8 @@ static itemPtr atom10_parse_entry(feedParserCtxtPtr ctxt, xmlNodePtr cur) {
 		if((cur->ns->href   && (nsh = (NsHandler *)g_hash_table_lookup(ns_atom10_ns_uri_table, (gpointer)cur->ns->href))) ||
 		   (cur->ns->prefix && (nsh = (NsHandler *)g_hash_table_lookup(atom10_nstable, (gpointer)cur->ns->prefix)))) {
 			
-			if(pf = nsh->parseItemTag)
+			pf = nsh->parseItemTag;
+			if(pf)
 				(*pf)(ctxt, cur);
 			cur = cur->next;
 			continue;
@@ -459,14 +481,14 @@ static itemPtr atom10_parse_entry(feedParserCtxtPtr ctxt, xmlNodePtr cur) {
 		/* check namespace of this tag */
 		if(!cur->ns->href) {
 			/* This is an invalid feed... no idea what to do with the current element */
-			debug1(DEBUG_PARSING, "element with no namespace found in atom feed %s!", node_get_title(ctxt->node));
+			debug1(DEBUG_PARSING, "element with no namespace found in atom feed (%s)!", cur->name);
 			cur = cur->next;
 			continue;
 		}
 		
 		
 		if(0 != xmlStrcmp(cur->ns->href, ATOM10_NS)) {
-			debug2(DEBUG_PARSING, "unknown namespace %s found in atom feed %s", cur->ns->href, node_get_title(ctxt->node));
+			debug1(DEBUG_PARSING, "unknown namespace %s found!", cur->ns->href);
 			cur = cur->next;
 			continue;
 		}
@@ -475,7 +497,7 @@ static itemPtr atom10_parse_entry(feedParserCtxtPtr ctxt, xmlNodePtr cur) {
 		if(func != NULL) {
 			(*func)(cur, ctxt, NULL);
 		} else {
-			debug2(DEBUG_PARSING, "unknown entry element \"%s\" found in atom feed\n", cur->name, node_get_title(ctxt->node));
+			debug1(DEBUG_PARSING, "unknown entry element \"%s\" found", cur->name);
 		}
 		
 		cur = cur->next;
@@ -493,7 +515,7 @@ static itemPtr atom10_parse_entry(feedParserCtxtPtr ctxt, xmlNodePtr cur) {
 static void atom10_parse_feed_author(xmlNodePtr cur, feedParserCtxtPtr ctxt, itemPtr ip, struct atom10ParserState *state) {
 	/* parse feed author */
 	gchar *author = atom10_parse_person_construct(cur);
-	ctxt->feed->metadata = metadata_list_append(ctxt->feed->metadata, "author", author);
+	ctxt->subscription->metadata = metadata_list_append(ctxt->subscription->metadata, "author", author);
 	g_free(author);
 	/* FIXME: make item parsing use this author if not specified elsewhere */
 }
@@ -508,7 +530,7 @@ static void atom10_parse_feed_category(xmlNodePtr cur, feedParserCtxtPtr ctxt, i
 	}
 	if(label != NULL) {
 		escaped = g_markup_escape_text(label, -1);
-		ctxt->feed->metadata = metadata_list_append(ctxt->feed->metadata, "category", escaped);
+		ctxt->subscription->metadata = metadata_list_append(ctxt->subscription->metadata, "category", escaped);
 		g_free(escaped);
 		xmlFree(label);
 	}
@@ -517,14 +539,14 @@ static void atom10_parse_feed_category(xmlNodePtr cur, feedParserCtxtPtr ctxt, i
 static void atom10_parse_feed_contributor(xmlNodePtr cur, feedParserCtxtPtr ctxt, itemPtr ip, struct atom10ParserState *state) {
 	/* parse feed contributors */
 	gchar *contributer = atom10_parse_person_construct(cur);
-	ctxt->feed->metadata = metadata_list_append(ctxt->feed->metadata, "contributor", contributer);
+	ctxt->subscription->metadata = metadata_list_append(ctxt->subscription->metadata, "contributor", contributer);
 	g_free(contributer);
 }
 
 static void atom10_parse_feed_generator(xmlNodePtr cur, feedParserCtxtPtr ctxt, itemPtr ip, struct atom10ParserState *state) {
 	gchar *ret, *version, *tmp = "", *uri;
 
-	ret = unhtmlize(common_utf8_fix(xmlNodeListGetString(cur->doc, cur->xmlChildrenNode, 1)));
+	ret = unhtmlize(xmlNodeListGetString(cur->doc, cur->xmlChildrenNode, 1));
 	if(ret != NULL && ret[0] != '\0') {
 		version = common_utf8_fix(xmlGetNsProp(cur, BAD_CAST"version", NULL));
 		if(version != NULL) {
@@ -540,7 +562,7 @@ static void atom10_parse_feed_generator(xmlNodePtr cur, feedParserCtxtPtr ctxt,
 			g_free(ret);
 			ret = tmp;
 		}
-		ctxt->feed->metadata = metadata_list_append(ctxt->feed->metadata, "feedgenerator", tmp);
+		ctxt->subscription->metadata = metadata_list_append(ctxt->subscription->metadata, "feedgenerator", tmp);
 	}
 	g_free(ret);
 }
@@ -556,8 +578,9 @@ static void atom10_parse_feed_id(xmlNodePtr cur, feedParserCtxtPtr ctxt, struct
 static void atom10_parse_feed_link(xmlNodePtr cur, feedParserCtxtPtr ctxt, struct atom10ParserState *state) {
 	gchar *href;
 	
-	if(href = atom10_parse_link(cur, ctxt, state)) {
-		feed_set_html_url(ctxt->feed, href);
+	href = atom10_parse_link(cur, ctxt, state);
+	if(href) {
+		feed_set_html_url(ctxt->feed, subscription_get_source(ctxt->subscription), href);
 		/* Set the default base to the feed's HTML URL if not set yet */
 		if (xmlNodeGetBase(cur->doc, xmlDocGetRootElement(cur->doc)) == NULL)
 			xmlNodeSetBase(xmlDocGetRootElement(cur->doc), href);
@@ -568,7 +591,8 @@ static void atom10_parse_feed_link(xmlNodePtr cur, feedParserCtxtPtr ctxt, struc
 static void atom10_parse_feed_logo(xmlNodePtr cur, feedParserCtxtPtr ctxt, struct atom10ParserState *state) {
 	gchar *logoUrl;
 	
-	if(logoUrl = atom10_parse_text_construct(cur, FALSE)) {
+	logoUrl = atom10_parse_text_construct(cur, FALSE);
+	if(logoUrl) {
 		/* FIXME: Verify URL is not evil... */
 		feed_set_image_url(ctxt->feed, logoUrl);
 		g_free(logoUrl);
@@ -578,8 +602,9 @@ static void atom10_parse_feed_logo(xmlNodePtr cur, feedParserCtxtPtr ctxt, struc
 static void atom10_parse_feed_rights(xmlNodePtr cur, feedParserCtxtPtr ctxt, struct atom10ParserState *state) {
 	gchar *rights;
 	
-	if(rights = atom10_parse_text_construct(cur, FALSE)) {
-		ctxt->feed->metadata = metadata_list_append(ctxt->feed->metadata, "copyright", rights);
+	rights = atom10_parse_text_construct(cur, FALSE);
+	if(rights) {
+		ctxt->subscription->metadata = metadata_list_append(ctxt->subscription->metadata, "copyright", rights);
 		g_free(rights);
 	}
 }
@@ -587,7 +612,8 @@ static void atom10_parse_feed_rights(xmlNodePtr cur, feedParserCtxtPtr ctxt, str
 static void atom10_parse_feed_subtitle(xmlNodePtr cur, feedParserCtxtPtr ctxt, struct atom10ParserState *state) {
 	gchar *subtitle;
 	
-	if(subtitle = atom10_parse_text_construct(cur, TRUE)) {
+	subtitle = atom10_parse_text_construct(cur, TRUE);
+	if(subtitle) {
 		feed_set_description(ctxt->feed, subtitle);
 		g_free(subtitle);
 	}
@@ -596,17 +622,20 @@ static void atom10_parse_feed_subtitle(xmlNodePtr cur, feedParserCtxtPtr ctxt, s
 static void atom10_parse_feed_title(xmlNodePtr cur, feedParserCtxtPtr ctxt, struct atom10ParserState *state) {
 	gchar *title;
 	
-	if(title = atom10_parse_text_construct(cur, FALSE)) {
-		node_set_title(ctxt->node, title);
-		g_free(title);
+	title = atom10_parse_text_construct(cur, FALSE);
+	if(title) {
+		if(ctxt->title)
+			g_free(ctxt->title);
+		ctxt->title = title;
 	}
 }
 
 static void atom10_parse_feed_updated(xmlNodePtr cur, feedParserCtxtPtr ctxt, struct atom10ParserState *state) {
 	gchar *timestamp;
 	
-	if(timestamp = common_utf8_fix(xmlNodeListGetString(cur->doc, cur->xmlChildrenNode, 1))) {
-		ctxt->feed->metadata = metadata_list_append(ctxt->feed->metadata, "contentUpdateDate", timestamp);
+	timestamp = common_utf8_fix(xmlNodeListGetString(cur->doc, cur->xmlChildrenNode, 1));
+	if(timestamp) {
+		ctxt->subscription->metadata = metadata_list_append(ctxt->subscription->metadata, "contentUpdateDate", timestamp);
 		ctxt->feed->time = parseISO8601Date(timestamp);
 		g_free(timestamp);
 	}
@@ -615,7 +644,6 @@ static void atom10_parse_feed_updated(xmlNodePtr cur, feedParserCtxtPtr ctxt, st
 /* reads a Atom feed URL and returns a new channel structure (even if
    the feed could not be read) */
 static void atom10_parse_feed(feedParserCtxtPtr ctxt, xmlNodePtr cur) {
-	int 			error = 0;
 	NsHandler		*nsh;
 	parseChannelTagFunc	pf;
 	atom10ElementParserFunc func;
@@ -641,7 +669,6 @@ static void atom10_parse_feed(feedParserCtxtPtr ctxt, xmlNodePtr cur) {
 	while(TRUE) {
 		if(xmlStrcmp(cur->name, BAD_CAST"feed")) {
 			g_string_append(ctxt->feed->parseErrors, "<p>Could not find Atom 1.0 header!</p>");
-			error = 1;
 			break;			
 		}
 		
@@ -653,9 +680,20 @@ static void atom10_parse_feed(feedParserCtxtPtr ctxt, xmlNodePtr cur) {
 				continue;
 			}
 			
-			if((cur->ns->href   && (nsh = (NsHandler *)g_hash_table_lookup(ns_atom10_ns_uri_table, (gpointer)cur->ns->href))) ||
-			   (cur->ns->prefix && (nsh = (NsHandler *)g_hash_table_lookup(atom10_nstable, (gpointer)cur->ns->prefix)))) {
-				if(pf = nsh->parseChannelTag)
+			/* check if supported namespace should handle the current tag 
+			   by trying to determine a namespace handler */
+			   
+			nsh = NULL;
+			
+			if(cur->ns->href)
+				nsh = (NsHandler *)g_hash_table_lookup(ns_atom10_ns_uri_table, (gpointer)cur->ns->href);
+			
+			if(cur->ns->prefix && !nsh)
+				nsh = (NsHandler *)g_hash_table_lookup(atom10_nstable, (gpointer)cur->ns->prefix);
+				
+			if(nsh) {
+				pf = nsh->parseChannelTag;
+				if(pf)
 					(*pf)(ctxt, cur);
 				cur = cur->next;
 				continue;
@@ -664,13 +702,13 @@ static void atom10_parse_feed(feedParserCtxtPtr ctxt, xmlNodePtr cur) {
 			/* check namespace of this tag */
 			if(!cur->ns->href) {
 				/* This is an invalid feed... no idea what to do with the current element */
-				debug1(DEBUG_PARSING, "element with no namespace found in atom feed %s!", node_get_title(ctxt->node));
+				debug1(DEBUG_PARSING, "element with no namespace found in atom feed (%s)!", cur->name);
 				cur = cur->next;
 				continue;
 			}
 
 			if(xmlStrcmp(cur->ns->href, ATOM10_NS)) {
-				debug2(DEBUG_PARSING, "unknown namespace %s found in atom feed %s", cur->ns->href, node_get_title(ctxt->node));
+				debug1(DEBUG_PARSING, "unknown namespace %s found in atom feed!", cur->ns->href);
 				cur = cur->next;
 				continue;
 			}
@@ -680,16 +718,15 @@ static void atom10_parse_feed(feedParserCtxtPtr ctxt, xmlNodePtr cur) {
 			if(func) {
 				(*func)(cur, ctxt, NULL);
 			} else if((xmlStrEqual(cur->name, BAD_CAST"entry"))) {
-				if(ctxt->item = atom10_parse_entry(ctxt, cur))
-					itemset_append_item(ctxt->itemSet, ctxt->item);
+				ctxt->item = atom10_parse_entry(ctxt, cur);
+				if(ctxt->item)
+					ctxt->items = g_list_append(ctxt->items, ctxt->item);
 			}
 			cur = cur->next;
 		}
 		
 		/* FIXME: Maybe check to see that the required information was actually provided (persuant to the RFC). */
 		/* after parsing we fill in the infos into the feedPtr structure */		
-		if(error)
-			ui_mainwindow_set_status_bar(_("There were errors while parsing the feed %s!"), node_get_title(ctxt->node));
 		
 		break;
 	}
@@ -729,6 +766,7 @@ feedHandlerPtr atom10_init_feed_handler(void) {
 		atom10_add_ns_handler(ns_cC_getRSSNsHandler());
 		atom10_add_ns_handler(ns_photo_getRSSNsHandler());
 		atom10_add_ns_handler(ns_pb_getRSSNsHandler());
+		atom10_add_ns_handler(ns_wfw_getRSSNsHandler());
 
 	}	
 	/* prepare feed handler structure */
