@@ -1,7 +1,7 @@
 /**
  * @file render.c generic XSLT rendering handling
  * 
- * Copyright (C) 2006 Lars Lindner <lars.lindner@gmx.net>
+ * Copyright (C) 2006-2007 Lars Lindner <lars.lindner@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -151,22 +151,37 @@ static themeColorPtr render_get_theme_color(const gchar *name, GdkColor themeCol
 	return tc;
 }
 
-static void render_get_theme_colors(void) {
+static void
+render_get_theme_colors (void)
+{
 	GtkWidget	*htmlview;
 	GtkStyle	*style;
+//	GdkColor	*color;
 	
-	htmlview = ui_mainwindow_get_active_htmlview();
-	style = gtk_widget_get_style(htmlview);
-	g_assert(NULL != style);
+	htmlview = liferea_htmlview_get_widget (ui_mainwindow_get_active_htmlview ());
+	style = gtk_widget_get_style (htmlview);
+	g_assert (NULL != style);
 
-	g_assert(NULL == themeColors);
-	themeColors = g_slist_append(themeColors, render_get_theme_color("GTK-COLOR-FG",    style->fg[GTK_STATE_NORMAL]));
-	themeColors = g_slist_append(themeColors, render_get_theme_color("GTK-COLOR-BG",    style->bg[GTK_STATE_NORMAL]));
-	themeColors = g_slist_append(themeColors, render_get_theme_color("GTK-COLOR-LIGHT", style->light[GTK_STATE_NORMAL]));
-	themeColors = g_slist_append(themeColors, render_get_theme_color("GTK-COLOR-DARK",  style->dark[GTK_STATE_NORMAL]));
-	themeColors = g_slist_append(themeColors, render_get_theme_color("GTK-COLOR-MID",   style->mid[GTK_STATE_NORMAL]));
-	themeColors = g_slist_append(themeColors, render_get_theme_color("GTK-COLOR-BASE",  style->base[GTK_STATE_NORMAL]));
-	themeColors = g_slist_append(themeColors, render_get_theme_color("GTK-COLOR-TEXT",  style->text[GTK_STATE_NORMAL]));
+	g_assert (NULL == themeColors);
+	themeColors = g_slist_append (themeColors, render_get_theme_color("GTK-COLOR-FG",    style->fg[GTK_STATE_NORMAL]));
+	themeColors = g_slist_append (themeColors, render_get_theme_color("GTK-COLOR-BG",    style->bg[GTK_STATE_NORMAL]));
+	themeColors = g_slist_append (themeColors, render_get_theme_color("GTK-COLOR-LIGHT", style->light[GTK_STATE_NORMAL]));
+	themeColors = g_slist_append (themeColors, render_get_theme_color("GTK-COLOR-DARK",  style->dark[GTK_STATE_NORMAL]));
+	themeColors = g_slist_append (themeColors, render_get_theme_color("GTK-COLOR-MID",   style->mid[GTK_STATE_NORMAL]));
+	themeColors = g_slist_append (themeColors, render_get_theme_color("GTK-COLOR-BASE",  style->base[GTK_STATE_NORMAL]));
+	themeColors = g_slist_append (themeColors, render_get_theme_color("GTK-COLOR-TEXT",  style->text[GTK_STATE_NORMAL]));
+	
+/*	color = NULL;
+	gtk_widget_style_get(GTK_WIDGET(mainwindow), "link-color", &color, NULL);
+	if(color)
+		themeColors = g_slist_append(themeColors, render_get_theme_color("GTK-COLOR-NORMAL-LINK", *color));
+	g_free(color);
+
+	color = NULL;	
+	gtk_widget_style_get(GTK_WIDGET(mainwindow), "visited-link-color", &color, NULL);
+	if(color)
+		themeColors = g_slist_append(themeColors, render_get_theme_color("GTK-COLOR-VISITED-LINK", *color));
+	g_free(color);*/
 }
 
 static gchar * render_set_theme_colors(gchar *css) {
@@ -181,35 +196,38 @@ static gchar * render_set_theme_colors(gchar *css) {
 	return css;
 }
 
-void render_remove_css_comments(void) {
-	gssize i, len = css->len;
-	gchar *str = css->str;
-	GString *newcss;
-
-	newcss = g_string_new(NULL);
-	i = 0;
-	while (i < len) {
-		if (i+1 < len && str[i] == '/' && str[i+1] == '*') {
-			i += 4;
-			while (i-1 < len && (str[i-2] != '*' || str[i-1] != '/'))
-				i++;
-			continue;
-		}
-		if (i+1 < len && str[i] == '/' && str[i+1] == '/') {
-			i += 2;
-			while (i < len && str[i] != '\n')
-				i++;
-			continue;
-		}
-		newcss = g_string_append_c(newcss, str[i]);
-		i++;
-	}
-	g_string_free(css, TRUE);
-	css = newcss;
+static void
+render_remove_css_comments (void)
+{
+ 	gssize i, len = css->len;
+ 	gchar *str = css->str;
+ 	GString *newcss;
+ 
+ 	newcss = g_string_new(NULL);
+ 	i = 0;
+ 	while (i < len) {
+ 		if (i+1 < len && str[i] == '/' && str[i+1] == '*') {
+ 			i += 4;
+ 			while (i-1 < len && (str[i-2] != '*' || str[i-1] != '/'))
+ 				i++;
+ 			continue;
+ 		}
+ 		if (i+1 < len && str[i] == '/' && str[i+1] == '/') {
+ 			i += 2;
+ 			while (i < len && str[i] != '\n')
+ 				i++;
+ 			continue;
+ 		}
+ 		newcss = g_string_append_c(newcss, str[i]);
+ 		i++;
+ 	}
+ 	g_string_free(css, TRUE);
+ 	css = newcss;
 }
 
-const gchar * render_get_css(void) {
-	
+const gchar *
+render_get_css (gboolean externalCss)
+{	
 	if(!css) {   
 		gchar	*styleSheetFile, *defaultStyleSheetFile, *adblockStyleSheetFile;
 		gchar	*font = NULL;
@@ -274,13 +292,28 @@ const gchar * render_get_css(void) {
 		}
 
 		g_free(adblockStyleSheetFile);
-
-		/* remove comments from the CSS to circumvent libgtkhtml2 bug */
-		render_remove_css_comments();
-
-		/* keep the CSS in memory to serve it as a part of each HTML output */
-		g_string_prepend(css, "<style type=\"text/css\">\n<![CDATA[\n");
-		g_string_append(css, "\n]]>\n</style>\n");
+		
+		/* remove comments from the CSS to circumvent libgtkhtml2 freezing bug */
+ 		render_remove_css_comments();
+		
+		if(externalCss) {
+			/* dump CSS to cache file and create a <style> tag to use the it */
+			gchar *filename = common_create_cache_filename("cache", "style", "css");
+			if(!g_file_set_contents(filename, css->str, -1, NULL))
+				g_warning("Cannot write temporary CSS file \"%s\"!", filename);
+			
+			g_string_free(css, TRUE);
+			
+			css = g_string_new("<style type=\"text/css\"> @import url(file://");
+			g_string_append(css, filename);
+			g_string_append(css, "); </style> ");
+			
+			g_free(filename);
+		} else {
+			/* keep the CSS in memory to serve it as a part of each HTML output */
+			g_string_prepend(css, "<style type=\"text/css\">\n<![CDATA[\n");
+			g_string_append(css, "\n]]>\n</style>\n");
+		}
 	}
 	
 	return css->str;
